<?php

namespace mageekguy\atoum\reports\asynchronous;

use
	mageekguy\atoum,
	mageekguy\atoum\exceptions,
	mageekguy\atoum\report\fields
;

class xunit extends atoum\reports\asynchronous
{
	const defaultTitle = 'atoum testsuite';

	protected $score = null;

	public function __construct(atoum\adapter $adapter = null)
	{
		parent::__construct();

		$this->setAdapter($adapter);

		if ($this->adapter->extension_loaded('libxml') === false)
		{
			throw new exceptions\runtime('libxml PHP extension is mandatory for xunit report');
		}
	}

	public function handleEvent($event, atoum\observable $observable)
	{
		$this->score = ($event !== atoum\runner::runStop) ? null : $observable->getScore();
		return parent::handleEvent($event, $observable);
	}

	protected function getTestedClasses()
	{
		$durations = $this->score->getDurations();
		$errors = $this->score->getErrors();
		$excepts = $this->score->getExceptions();
		$fails = $this->score->getFailAssertions();

		$filterClass = function ($element) use (& $clname) { return ($element['class'] == $clname); };
		$extractClasses = function($list) use (& $clname, & $classes, $durations, $errors, $excepts, $fails, $filterClass) {
			foreach ($list as $entry)
			{
				$clname = $entry['class'];

				if (isset($classes[$clname]) === false)
				{
					$classes[$clname] = array(
						'errors' => array_filter($errors, $filterClass),
						'excepts' => array_filter($excepts, $filterClass),
						'fails' => array_filter($fails, $filterClass),
						'durations' => array_filter($durations, $filterClass)
					);
				}
			}
		};

		$classes = array();
		$extractClasses($durations);
		$extractClasses($errors);
		$extractClasses($excepts);
		$extractClasses($fails);

		return $classes;
	}

	public function build($event)
	{
		$this->string = '';

		if ($event === atoum\runner::runStop)
		{
			$this->title = $this->title ?: self::defaultTitle;

			$document = new \DOMDocument('1.0', 'UTF-8');
			$document->formatOutput = true;
			$document->appendChild($root = $document->createElement('testsuites'));
			$root->setAttribute('name', $this->title);
			$classes = $this->getTestedClasses();

			foreach ($classes as $name => $class)
			{
				$testCase = null;
				$clname = $package = $name;
				$antiSlashOffset = strrpos($clname, '\\');
				if ($antiSlashOffset !== false)
				{
					$clname = substr($clname, $antiSlashOffset + 1);
					$package = substr($name, 0, $antiSlashOffset);
				}

				$root->appendChild($testSuite  = $document->createElement('testsuite'));

				$testSuite->setAttribute('name', $clname);
				$testSuite->setAttribute('package', $package);
				$testSuite->setAttribute('tests', sizeof($class['durations']) + ($fails = sizeof($class['fails'])) + ($errors = sizeof($class['excepts']) + sizeof($class['errors'])));
				$testSuite->setAttribute('failures', $fails);
				$testSuite->setAttribute('errors', $errors);

				$time = 0;

				foreach ($class['durations'] as $duration)
				{
					$time += $duration['value'];

					static::getTestCase($document, $testSuite, $name, $duration['method'], $duration['value'], $duration['path']);
				}

				$testSuite->setAttribute('time', $time);

				foreach ($class['errors'] as $error)
				{
					$testCase = static::getTestCase($document, $testSuite, $name, $error['method'], 0, $error['file']);
					$testCase->appendChild($xError = $document->createElement('error'));

					$xError->setAttribute('type', $error['type']);
					$xError->appendChild($document->createCDATASection($error['message']));
				}

				foreach ($class['fails'] as $fail)
				{
					$testCase = static::getTestCase($document, $testSuite, $name, $fail['method'], 0, $fail['file']);
					$testCase->appendChild($xFail = $document->createElement('failure'));

					$xFail->setAttribute('type', 'Failure');
					$xFail->setAttribute('message', $fail['asserter']);

					$xFail->appendChild($document->createCDATASection($fail['fail']));
				}

				foreach ($class['excepts'] as $exc)
				{
					$testCase = static::getTestCase($document, $testSuite, $name, $exc['method'], 0, $exc['file']);
					$testCase->appendChild($xError = $document->createElement('error'));

					$xError->setAttribute('type', 'Exception');
					$xError->appendChild($document->createCDATASection($exc['value']));
				}
			}

			$this->string = $document->saveXML();
		}

		return $this;
	}

	private static function getTestCase(\DOMDocument $document, \DOMElement $testSuite, $class, $method, $time, $path)
	{
		if (($testCase = static::findTestCase($document, $class, $method)) === null)
		{
			$testCase = $document->createElement('testcase');
			$testCase->setAttribute('name', $method);
			$testCase->setIdAttribute('name', true);
			$testCase->setAttribute('time', $time);
			$testCase->setAttribute('file', $path);
			$testCase->setAttribute('classname', $class);

			$testSuite->appendChild($testCase);
		}

		return $testCase;
	}

	private static function findTestCase(\DOMDocument $document, $class, $method)
	{
		$xpath = new \DOMXPath($document);
		$query = $xpath->query("//testcase[@classname='$class' and @name='$method']");

		if ($query->length > 0)
		{
			return $query->item(0);
		}

		return null;
	}
}
